//( lifneuron.hpp
#ifndef LIFNEURON_HPP
#define LIFNEURON_HPP


#include <vector>

using namespace std;

#include "synapse.hpp"

#include "firingneuron.hpp"

//( LifNeuron

/**
 * Leak, integrate and fire neuron
 *
 * @author Thomas
 **/
class LifNeuron :
  public FiringNeuron
{
public:
  /**
   *
   * Constructor
   *
   * @param   resetPotential       reset potential of this neuron (Volt)
   * @param   restingPotential     resting potential of this neuron (Volt)
   * @param   firingThreshold      firing threshold potential of this neuron (Volt)
   * @param   membraneCapacitance  membrane capacitance of this neuron (Farad)
   * @param   leakConductance      membrane leak conductance of this neuron (Siemens)
   * @param   refractoryPeriod     refactory period of this neuron (Second)
   *
   *
   **/
  LifNeuron(double resetPotential, double restingPotential,
	    double firingThreshold, double membraneCapacitance,
	    double leakConductance, double refractoryPeriod,
	    bool hebbLearning = false,
	    bool presynapticLearning = false, bool postsynapticLearning = false);



  /**
   * Default destructor
   *
   **/
  ~LifNeuron();


public:
  /**
   *
   *
   * @return  the current potential of this neuron (Ampere)
   *
   **/
  double getCurrentPotential() const;

  /**
   *
   * Set an input intensity for this neuron for testing purpose only
   *
   * @param   intensity  intensity to input fir this neuron (ampere)
   *
   * @return  void
   *
   **/
  void setInputTestingIntensity(double intensity);


  /**
   *
   *
   * @return  true if the neuron emitted a spike
   *
   **/
  bool isFiring() const;

  /**
   *
   * Add a receptor to this neuron
   *
   * @param   synapse  synapse using the post synaptic receptor
   *
   * @return  void
   *
   **/
  void addReceptor(Synapse * synapse);

  /**
   *
   * Get a receptor of a particular type
   *
   * @param   receptorType  type of the receptor looked for
   *
   * @return  a pointer on a corresponding type, null if not found
   *
   **/
  Synapse * getReceptor(const Synapse::ReceptorType receptorType);



public:
  /**
   *
   * Define an array of intensity
   *
   **/
  typedef vector<double> IntensityArray;

  /**
   *
   * Set an array to store the input intensity of this neurons for statistics
   *
   * @param   intensityArray  an array to store the information
   *
   * @return  void
   *
   **/
  void setIntensityStatisticArray(IntensityArray * intensityArray);

public:
  /**
   *
   * do the hebbian learning on all the synaptic connections of the neuron
   *
   * @return  void
   *
   **/
  void computeRateBasedLearning();

  /**
   *
   * given synapse and neurons stats compute the change in weight
   *
   * @param weight   the synapse present weight
   * @param afferentRate  the present mean firing rate of afferent neuron
   * @param efferentRate  the present mean firing rate of efferent neuron
   *
   * @return  void
   *
   **/
  double computeRateBasedLearning(double weight, double afferentRate, double efferentRate);


  /**
   *
   * ...
   *
   *
   * @return  value of the presynaptic function of the neuron
   *
   **/
  double getPresynapticFunctionValue();


  /**
   *
   * ...
   *
   *
   * @return  value of the postsynaptic function
   *
   **/
  double getPostsynapticFunctionValue();

  /**
   *
   *  added by Xav
   *
   *
   * @return  the time of the last emitted spike
   *
   **/
  double getLastSpikeTime();

  /**
   *
   *  added by Xav
   *
   * @param   the time between pre and postsynaptic spikes
   *
   * @return  value of the learning windows
   *
   **/
  double learningWindow(double timeGap);


  list<double> getSpikeTimeList() {return spikeTimeList;};


protected:
  /**
   *
   * Fill the statistic array with the intensity coming as input for
   * this neuron
   *
   *
   * @return  void
   *
   **/
  void fillIntensityStatisticArray();

  /**
   *
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return the current input intensity of this neuron (synapse + a
   * testing intensity) (Ampere)
   *
   **/
  double getCurrentInputIntensity(RungeKutta rungeKutta) const;

  /**
   *
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the current input intensity of this neuron from the synapse
   *
   **/
  double getCurrentSynapticIntensity(RungeKutta rungeKutta) const;

  /**
   *
   *
   * @return  true if the neuron is in a state where it can fire, else false
   *
   **/
  bool canFire() const;

  /**
   *
   * Generate a spike for this neuron
   *
   *
   * @return  void
   *
   **/
  void generateSpike();

  /**
   *
   * Compute the derivate of the equation modeling the behaviour of
   * this neurons
   *
   * @param   currentValue  the last value of the function
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the result of the computation
   *
   **/
  double computeDerivate(double currentValue, RungeKutta rungeKutta);


protected:
  /**
   *
   * Compute one more step for this object
   *
   *
   * @return  void
   *
   **/
  void computeOneStep();


private:
  /**
   *
   * Defines the type of the list of the spike of this neuron
   *
   **/
  typedef list<double> SpikeTimeList;

  /**
   *
   * List of the spike of this neuron from the beginning of the
   * simulation
   *
   **/
  SpikeTimeList spikeTimeList;


public:
  /**
   *
   * returns the firing times of the neuron for a certain period before the present time
   *
   * @param   timeWindow    the time window (in s)
   * @param   lastFiringTimes   the list of firing times included within the time window
   *
   * @return  void
   *
   **/
  void getFiringTimes(double timeWindow, SpikeTimeList & lastFiringTimes);

protected:
  /**
   *
   * compute the number of spikes emitted since present time - timeGap
   *
   * @param   timeGap     width of time interval
   *
   * @return  the number of spikes
   *
   **/
  int getNbOfRecentSpikes(double timeGap);


private:
  /**
   *
   * class used to compute the presynaptic function value for spike-dependant learning
   *
   **/
  class PresynapticFunction:
    public TimeDependantObject
  {
  public:
    /**
     * Constructor
     *
     * @param parent   the parent neuron of this function
     **/
    PresynapticFunction(LifNeuron * parent);

  protected:
    /**
     *
     * Compute one more step for this object
     *
     *
     * @return  void
     *
     **/
    void computeOneStep();

    /**
     *
     * Compute the derivate of this function
     *
     *
     * @param   currentValue  the last value of the function
     * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
     *
     * @return  the result of the computation
     *
     **/
    double computeDerivate(double currentValue, RungeKutta rungeKutta);

  private:
    /**
     *
     * Synapse using this instance
     *
     **/
    LifNeuron * parent;
  };

private:
  /**
   *
   * class used to compute the postsynaptic function value for spike-dependant learning
   *
   **/
  class PostsynapticFunction:
    public TimeDependantObject
  {
  public:
    /**
     * Constructor
     *
     * @param parent   the parent neuron of this function
     **/
    PostsynapticFunction(LifNeuron * parent);

  protected:
    /**
     *
     * Compute one more step for this object
     *
     *
     * @return  void
     *
     **/
    void computeOneStep();

    /**
     *
     * Compute the derivate of this function
     *
     *
     * @param   currentValue  the last value of the function
     * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
     *
     * @return  the result of the computation
     *
     **/
    double computeDerivate(double currentValue, RungeKutta rungeKutta);

  private:
    /**
     *
     * Synapse using this instance
     *
     **/
    LifNeuron * parent;
  };

protected:
  /**
   *
   * a presynaptic function for spike-dependant learning
   *
   **/
  PresynapticFunction * presynapticFunction;

protected:
  /**
   *
   * a presynaptic function for spike-dependant learning
   *
   **/
  PostsynapticFunction * postsynapticFunction;

protected:
  /**
   *
   * Define the type vector of list of synapse using the same type of receptors
   *
   **/
  typedef vector<Synapse *> SynapseList;

  /**
   *
   * Vector of list of synapse using the same type of receptors
   *
   **/
  SynapseList synapseList;

protected:
  /**
   *
   * Array used to store statistics about the input intensity
   *
   **/
  IntensityArray * intensityStatisticArray;

protected:
  /**
   *
   * Reset potential of this neuron (Volt)
   *
   **/
  double resetPotential;

  /**
   *
   * Resting potential of this neuron (Volt)
   *
   **/
  double restingPotential;

  /**
   *
   * Firing threshold potential of this neuron (Volt)
   *
   **/
  double firingThreshold;

  /**
   *
   * Membrane capacitance of this neuron (Farad)
   *
   **/
  double membraneCapacitance;

  /**
   *
   * Membrane leak conductance of this neuron (Siemens)
   *
   **/
  double leakConductance;

  /**
   *
   * Refactory period of this neuron (Second)
   *
   **/
  double refractoryPeriod;

  /**
   *
   * Input intensity in this neuron for testing purpose only
   *
   **/
  double inputTestingIntensity;


  /**
   *
   * true if the neuron is implicated in a rate-based hebbian learning
   *
   **/
  bool hebbLearning;

  /**
   *
   * true if the neuron is involved in spike-based learning as a pre-synaptic neuron
   *
   **/
  bool presynapticLearning;

  /**
   *
   * true if the neuron is involved in spike-based learning as a post-synaptic neuron
   *
   **/
  bool postsynapticLearning;

  /**
   *
   * used to store the amount of synaptic change in order to proceed regularly to re-equilibration of the network
   *
   **/
  double equilibrium;

}
;

//) LifNeuron


#endif

//) lifneuron.hpp
