//( neuralnetwork.cpp


#include <algorithm>
#include <math.h>
#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include "math.hpp"
#include <fstream>
#include <iostream>

#include "parameters.hpp"
#include "neuralnetwork.hpp"


//( NeuralNetwork


PyramidalCell NeuralNetwork::pyramidalCellModel;
//Interneuron NeuralNetwork::interneuronModel;
//PoissonNeuron NeuralNetwork::noiseNeuronModel;
MyLifNeuron NeuralNetwork::myLifNeuronModel;

//int argc=1;
//char **argv;

NeuralNetwork::NeuralNetwork() :
        firingPositions(2001,0)
{
    numberOfConnections = 0;
    currentAverage = false;

    resetFiringPositions();

}


NeuralNetwork::~NeuralNetwork()
{
    for (SynapsesVector::iterator iterator = synapsesVector.begin();
    iterator != synapsesVector.end();
    iterator++)
    {
        delete (*iterator);
    }
    synapsesVector.clear();

    for (NeuronsVector::iterator iterator = neuronsVector.begin();
    iterator != neuronsVector.end();
    iterator++)
    {
        delete (*iterator);
    }
    neuronsVector.clear();

}


void NeuralNetwork::createNeurons(const int numberOfNeurons, const FiringNeuron & model,
                                  NeuronsVector & neuronsLayer, bool noiseFlag,
                                  bool hebbLearning,
                                  bool presynapticLearning, bool postsynapticLearning)
{
    for (int index = 0;
         index < numberOfNeurons;
         index++)
    {
        FiringNeuron * newNeuron = model.createNeuron(hebbLearning, presynapticLearning, postsynapticLearning);
        newNeuron->setNoise(noiseFlag);
        neuronsLayer.push_back(newNeuron);
        neuronsVector.push_back(newNeuron);
    }
}


void NeuralNetwork::computeOneStep()
{
    resetFiringPositions();

    for (SynapsesVector::iterator iterator = synapsesVector.begin();
    iterator != synapsesVector.end();
    iterator++)
    {
        (*iterator)->nextStep();
        //cout << &(*iterator) << endl;
    }

    NeuronsVector::iterator iterator = neuronsVector.begin();
    for(int vectorIndex = 0;
        vectorIndex<neuronsVector.size();
        vectorIndex++)
    {
        (*iterator)->nextStep();
        iterator++;
    }

    NeuronsVector & mainVector = getMainNeuronsVector();
    iterator = mainVector.begin();
    int firingCounter = 1;
    for(int vectorIndex = 0;
        vectorIndex<mainVector.size();
        vectorIndex++)
    {
        if((*iterator)->isFiring())
        {
            firingPositions[firingCounter] = vectorIndex;
            firingCounter ++;
        }
        iterator++;
    }
    firingPositions[0] = firingCounter-1;
    //   std::cout << firingPositions[0] << " || ";
}

void NeuralNetwork::indexLayer(NeuronsVector & neuronsLayer, double neuronsRangeDistribution)
{
    int numberOfNeurons = neuronsLayer.size();
    for (int index  = 0;
         index < numberOfNeurons;
         index++)
    {
        neuronsLayer[index]->setPositionIndex((index * neuronsRangeDistribution)
                                              / numberOfNeurons);
    }
}

const NeuralNetwork::NeuronsVector * NeuralNetwork::getNeuronsVector() const
{
    return &neuronsVector;
}

const NeuralNetwork::SynapsesVector * NeuralNetwork::getSynapsesVector() const
{
    return &synapsesVector;
}

void NeuralNetwork::createOneConnections(NeuronsVector & neuronsVectorOrigin,
                                         NeuronsVector & neuronsVectorTarget,
                                         const Synapse::ReceptorType receptorType,
                                         double weight)
{

    assert(neuronsVectorOrigin.size() == neuronsVectorTarget.size());

    LifNeuron * referenceNeuron = static_cast<LifNeuron *>(*(neuronsVectorTarget.begin()));
    Synapse * referenceSynapse = referenceNeuron->getReceptor(receptorType);

    NeuronsVector::iterator targetIterator = neuronsVectorTarget.begin();
    for (NeuronsVector::iterator originIterator = neuronsVectorOrigin.begin();
    originIterator != neuronsVectorOrigin.end();
    originIterator++)
    {
        Synapse * synapse = static_cast<LifNeuron *>(*targetIterator)->getReceptor(receptorType);
        if (synapse == 0)
        {
            // 	  cout << "Warning: can't create one for one connections" << endl;
            return;
        }
        if (((*targetIterator) != referenceNeuron) && (synapse == referenceSynapse) )
        {
            // 	  cout << "Warning: the same synapse appears for a one<->one connection..." << endl;
        }

        addConnection((*originIterator),
                      static_cast<LifNeuron *>(*targetIterator),
                      synapse, weight);
        targetIterator++;
    }
    numberOfConnections += neuronsVectorOrigin.size();
}

void NeuralNetwork::createConnections(NeuronsVector & neuronsVectorOrigin,
                                      NeuronsVector & neuronsVectorTarget,
                                      Synapse::ReceptorType receptorType,
                                      double weight)
{
    Synapse * synapse = static_cast<LifNeuron *>(*(neuronsVectorTarget.begin()))->getReceptor(receptorType);

    if (synapse != 0)
    {
        for (NeuronsVector::iterator originIterator = neuronsVectorOrigin.begin();
        originIterator != neuronsVectorOrigin.end();
        originIterator++)
        {
            addConnection(*originIterator, 0, synapse, weight);
        }

        for (NeuronsVector::iterator targetIterator = neuronsVectorTarget.begin();
        targetIterator != neuronsVectorTarget.end();
        targetIterator++)
        {
            addConnection(0, static_cast<LifNeuron *>(*targetIterator), synapse, weight);
            if (static_cast<LifNeuron *>(*targetIterator)->getReceptor(receptorType)!= synapse)
            {
                // 	      cout << "Warning: this is not the same synapse for equivalent connections"
                // 		   << endl;
                return;
            }
        }
        numberOfConnections += neuronsVectorOrigin.size() * neuronsVectorTarget.size();
    }
    else
    {
        //       cout << "Warning: can't create full connections" << endl;
    }
}


void NeuralNetwork::createRandomConnections(NeuronsVector & neuronsVectorOrigin,
                                            NeuronsVector & neuronsVectorTarget,
                                            const Synapse::ReceptorType receptorType,
                                            double weightMin, double weightMax)
{

    LifNeuron * referenceNeuron = static_cast<LifNeuron *>(*(neuronsVectorTarget.begin()));
    Synapse * referenceSynapse = referenceNeuron->getReceptor(receptorType);


    double weight;
    for (NeuronsVector::iterator originIterator = neuronsVectorOrigin.begin();
    originIterator != neuronsVectorOrigin.end();
    originIterator++)
    {
        for (NeuronsVector::iterator targetIterator = neuronsVectorTarget.begin();
        targetIterator!= neuronsVectorTarget.end();
        targetIterator++)
        {
            Synapse * synapse = static_cast<LifNeuron *>(*targetIterator)->getReceptor(receptorType);
            if (synapse == 0)
            {
                cout << "Warning: can't create one for one connections" << endl;
                return;
            }
            if (((*targetIterator) != referenceNeuron) && (synapse == referenceSynapse) )
            {
                cout << "Warning: the same synapse appears for a one<->one connection..." << endl;
            }

            weight=weightMin+ (double)rand()*(weightMax-weightMin)/RAND_MAX;
            addConnection((*originIterator), static_cast<LifNeuron *>(*targetIterator), synapse, weight);
        }
    }
    numberOfConnections += neuronsVectorOrigin.size()*neuronsVectorTarget.size();
}




void NeuralNetwork::createPostsynapticConnections(NeuronsVector & neuronsLayer,
                                                  Synapse::ReceptorType receptorType,
                                                  bool uniqueSynapse,
                                                  bool externConnection)
{
    Synapse * synapse;
    switch (receptorType)
    {
    case Synapse::NMDAReceptor :
        synapse = new SynapseNMDA();
        break;
    case Synapse::AMPAReceptor :
        synapse = new SynapseAMPA();
        break;
    case Synapse::GABAReceptor :
        synapse = new SynapseGABA();
        break;
    }

    Synapse * newSynapse = 0;
    if (uniqueSynapse)
    {
        newSynapse = synapse->createSynapse();
        newSynapse->setRecurrent(!externConnection);
        synapsesVector.push_back(newSynapse);
    }

    for (NeuronsVector::const_iterator iterator = neuronsLayer.begin();
    iterator != neuronsLayer.end();
    iterator++)
    {
        if (!uniqueSynapse)
        {
            newSynapse = synapse->createSynapse();
            newSynapse->setRecurrent(!externConnection);
            synapsesVector.push_back(newSynapse);
        }
        newSynapse->setRecurrent(!externConnection);
        static_cast<LifNeuron *>(*iterator)->addReceptor(newSynapse);
    }
    delete synapse;
}

void NeuralNetwork::addConnection(FiringNeuron * presynapticNeuron,
                                  LifNeuron * postsynapticNeuron,
                                  Synapse * synapse, double weight)
{
    assert(synapse != 0);

    //cout << weight << "\t" << minimumSynapseWeight <<endl;

    if (weight > minimumSynapseWeight)
    {
        synapse->addConnection(presynapticNeuron, postsynapticNeuron, weight);
    }
    else
    {
        cout << "poids trop petiot !" <<endl;
    }

}

void NeuralNetwork::printRaster(ostream & output, int time)
{
    bool tOK = false;
    saveDataOKSemaphore->acquire(1);
    tOK = saveDataOK;
    saveDataOKSemaphore->release(1);

    if(useSPIKEdisplay==1)
    {
        StoreDisplaySemaphore->acquire(1);                      // acquiring memory space
    }

    for(int index = 0;index < firingPositions[0];index++)
    {
        if(tOK && saveLIFspikes)
        {
            output << time << firingPositions[index+1] << "    " << endl;
        }
        if(useSPIKEdisplay==1)
        {
            DisplaySpikeData->push_back(firingPositions[index+1]);
            DisplayTimeData->push_back(time);
        }
    }
    if(useSPIKEdisplay==1)
    {
        StoreDisplaySemaphore->release(1);                      // releasing memory space
    }

    if(saveLIFspikes)
    {
        output.flush();
    }

    edlutSemaphore->acquire(1);
//    eSim->AddData(firingPositions,time);
    eSim->AddDataForSorting(firingPositions,time);
    edlutSemaphore->release(1);
}

void NeuralNetwork::initPostFiringPositions(NeuronsVector & afferentVector, NeuralNetwork & efferentNetwork)
{
    NeuronsVector::iterator iterator = afferentVector.begin();
    for(int index = 0;
        index < afferentVector.size();
        index++)
    {
        (*iterator)->initPostSynapticFirings(efferentNetwork.getFiringPositionsVector());
        iterator ++;
    }
}


void NeuralNetwork::resetFiringPositions()
{
    firingPositions[0] = 0;
    //    cout << "taille du vecteur : " << firingPositions.size() << endl;
}


FiringNeuron::FiringPositions & NeuralNetwork::getFiringPositionsVector()
{
    return firingPositions;
}


void NeuralNetwork::createRecordedConnections(NeuronsVector & neuronsVectorOrigin, NeuronsVector & neuronsVectorTarget, const Synapse::ReceptorType receptorType, const string & fileName, int originSize, int targetSize)
{
    assert( (neuronsVectorOrigin.size() == originSize) && (neuronsVectorTarget.size() == targetSize) );
    ifstream connections(fileName.c_str());

    NeuronsVector::iterator originIterator = neuronsVectorOrigin.begin();
    for (int originIndex = 0;
         originIndex < neuronsVectorOrigin.size();
         originIndex++)
    {
        NeuronsVector::iterator targetIterator = neuronsVectorTarget.begin();
        for (int targetIndex = 0;
             targetIndex < neuronsVectorTarget.size();
             targetIndex++)
        {
            double weight;
            connections >> weight;
            Synapse * synapse = static_cast<LifNeuron *>(*targetIterator)->getReceptor(receptorType);
            assert(synapse!=0);
            addConnection((*originIterator),
                          static_cast<LifNeuron *>(*targetIterator),
                          synapse, weight);
            targetIterator++;
        }
        originIterator++;
    }
    //Modif boiteuse ?
    connections.close();
}

//) NeuralNetwork


//) neuralnetwork.cpp
