//( timedependantobject.hpp

#ifndef TIMEDEPENDANTOBJECT_HPP
#define TIMEDEPENDANTOBJECT_HPP

#include <vector>
#include <boost/date_time/posix_time/posix_time.hpp>



//( TimeDependantObject

/**
 * Object dependant of the time
 *
 * @author Thomas
 **/
class TimeDependantObject
{
public:
  /**
   * Default constructor
   *
   **/
  TimeDependantObject();

  /**
   * Default destructor
   *
   **/
  virtual ~TimeDependantObject();


public:
  /**
   *
   * Compute the next step for this object
   *
   *
   * @return  void
   *
   **/
  void nextStep();


public:
  /**
   *
   *
   * @return  the time spend from the beginning of the simulation (Second)
x   *
   **/
  double getCurrentTime() const;


public:
  /**
   *
   * Start a chrono
   *
   *
   * @return  void
   *
   **/
  void startChrono();

  /**
   *
   * Get the current value of the chrono
   *
   *
   * @return  the last value measured by the chrono (second)
   *
   **/
  double getChrono() const;


public:
  /**
   *
   * Reset the time counter
   *
   *
   * @return  void
   *
   **/
  void resetTime();

  /**
   *
   * Set the duration of one step of computing in the program
   *
   * @param   timeStep  duration of a step (Second)
   *
   * @return  void
   *
   **/
  void setTimeStep(double timeStep);

  /**
   *
   *
   * @return  the current time step
   *
   **/
  double getTimeStep();


protected:
  /**
   *
   * Compute one more step for this object
   *
   *
   * @return  void
   *
   **/
  virtual void computeOneStep() = 0;


public:
  /**
   *
   * The different step of the Runge Kutta algorithm
   *
   **/
  typedef enum {
    none,
    k1Step,
    k2Step
  } RungeKutta;

  /**
   *
   *
   * @return the current value of the x function of the NMDA
   * receptor
   *
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   **/
  double getCurrentValue(RungeKutta rungeKutta = none) const;


protected:
  /**
   *
   * Compute the next step of this object using the Euler algorithm
   *
   * @param   timeStep   the time interval we want to compute for (in Second)
   *
   * @return  the result of the calculation
   *
   **/
  double computeNextStepEuler(double timeStep);

  /**
   *
   * Compute the next step of this object using the Runge-Kutta
   * algorithm (second order)
   *
   * http://nacphy.physics.orst.edu/ComPhys/DIFFEQ/mydif2/node5.html
   *
   * @param   timeStep   the time interval we want to compute for (in Second)
   *
   * @return  the result of the calculation
   *
   **/
  double computeNextStepRK2(double timeStep);

  /**
   *
   * Compute the derivate of the equation modeling the behaviour of
   * this object
   *
   * @param   currentValue  the last value of the function
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the result of the computation
   *
   **/
  virtual double computeDerivate(double currentValue, RungeKutta rungeKutta = none);


protected:
  /**
   *
   *
   * @return  the current system time in second
   *
   **/
  double getCurrentSystemTime() const;


protected:
  /**
   *
   * The number of step spent from the begining of the simulation
   *
   **/
  int currentStep;


protected:
  /**
   *
   * Duration of a step in the program
   *
   **/
  double timeStep;

  /**
   *
   * Variable used to measure the time
   *
   **/
  double chrono;


private:
  /**
   *
   * Runge-Kutta variable
   *
   **/
  double k1;

  /**
   *
   * Runge-Kutta variable
   *
   **/
  double k2;

  /**
   *
   * Value before this step
   *
   **/
  double lastValue;

protected:
  /**
   *
   * Current value of this object
   *
   **/
  double currentValue;


protected:
  /**
   *
   * start value of clock time
   *
   **/
  boost::posix_time::ptime preciseTime;

}
;

//) TimeDependantObject


#endif

//) timedependantobject.hpp
